/**
 ****************************************************************************************
 *
 * @file app_f000.c
 *
 * @brief findt Application Module entry point
 *
 * @auth  gang.cheng
 *
 * @date  2016.05.31
 *
 * Copyright (C) Beken 2009-2016
 *
 *
 ****************************************************************************************
 */
#ifndef APP_UART_TP_H_
#define APP_UART_TP_H_
/**
 ****************************************************************************************
 * @addtogroup APP
 * @ingroup RICOW
 *
 * @brief f000 Application Module entry point
 *
 * @{
 ****************************************************************************************
 */
/*
 * INCLUDE FILES
 ****************************************************************************************
 */

#include "rwip_config.h" // SW configuration

#include <stdint.h>  // Standard Integer Definition
#include "ke_task.h" // Kernel Task Definition

/*
 * STRUCTURES DEFINITION
 ****************************************************************************************
 */

/// f000s Application Module Environment Structure
struct app_uart_tp_env_tag
{
    /// Connection handle
    uint8_t conidx;
    /// Current Battery Level
    uint8_t uart_tx_payload;
};
/*
 * GLOBAL VARIABLES DECLARATIONS
 ****************************************************************************************
 */

/// f000s Application environment
extern struct app_uart_tp_env_tag app_uart_tp_env;

/// Table of message handlers
extern const struct ke_state_handler app_uart_table_handler;
/*
 * FUNCTIONS DECLARATION
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 *
 * f000s Application Functions
 *
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @brief Initialize f000s Application Module
 ****************************************************************************************
 */
void app_uart_tp_init(void);
/**
 ****************************************************************************************
 * @brief Add a f000 Service instance in the DB
 ****************************************************************************************
 */
void app_uart_add_uarts(void);
/**
 ****************************************************************************************
 * @brief Enable the f000 Service
 ****************************************************************************************
 */
void app_uart_enable_prf(uint8_t conidx);
/**
 ****************************************************************************************
 * @brief Send a Battery level value
 ****************************************************************************************
 */
void app_uart_send_lvl(uint8_t batt_lvl);

void app_uart_tx_send_payload(uint8_t *buf, uint8_t len);
#endif // APP_UART_TP_H_
