/**
 ****************************************************************************************
 *
 * @file led_switch_task.h
 *
 * @brief Header file - LED_SWITCH_TASK.
 *
 * Copyright (C) RivieraWaves 2009-2016
 *
 *
 ****************************************************************************************
 */

#ifndef LED_SWITCH_TASK_H_
#define LED_SWITCH_TASK_H_

/**
 ****************************************************************************************
 * @addtogroup LED Switch Task
 * @ingroup LED_SWITCH
 * @brief LED Switch Service Server Task
 * @{
 ****************************************************************************************
 */

/*
 * INCLUDE FILES
 ****************************************************************************************
 */

#include <stdint.h>
#include "rwip_task.h" // Task definitions
#include "led_switch.h"
#include "prf_types.h"


/*
 * DEFINES
 ****************************************************************************************
 */



/// Messages for LED Switch Service Server
enum
{
    ///Set the value of an attribute - Request
    LED_SWITCH_SET_VALUE_REQ = TASK_FIRST_MSG(TASK_ID_LED_SWITCH),
    ///Set the value of an attribute - Response
    LED_SWITCH_SET_VALUE_RSP,    
    /// Peer device request to get profile attribute value
    LED_SWITCH_VALUE_REQ_IND,
    /// Peer device confirm value of requested attribute
    LED_SWITCH_VALUE_CFM,
};

/*
 * API MESSAGES STRUCTURES
 ****************************************************************************************
 */

/// Parameters for the database creation
struct led_switch_db_cfg
{
    /// Database configuration @see enum led_switch_features
    uint16_t features;
};


///Set the value of an attribute - Request
struct led_switch_set_value_req
{
    /// Value to Set
    uint8_t value;
    /// Value length
    uint8_t length;
    /// Value data
    uint8_t data[__ARRAY_EMPTY];
};

///Set the value of an attribute - Response
struct led_switch_set_value_rsp
{
    /// Value Set
    uint8_t value;
    /// status of the request
    uint8_t status;
};

/// Peer device request to get profile attribute value
struct led_switch_value_req_ind
{
    /// Requested value
    uint8_t value;
};

/// Peer device  value of requested attribute
struct led_switch_value_cfm
{
    /// Requested value
    uint8_t value;
    /// Value length
    uint8_t length;
    /// Value data
    uint8_t data[__ARRAY_EMPTY];
};

/// @} LED_SWITCH_TSK
#endif // LED_SWITCH_TASK_H_
