/**
 ****************************************************************************************
 *
 * @file f000_task.h
 *
 * @brief Header file - Battery Service Server Role Task.
 *
 * Copyright (C) RivieraWaves 2009-2015
 *
 *
 ****************************************************************************************
 */


#ifndef _UART_TP_TASK_H_
#define _UART_TP_TASK_H_


#include "rwprf_config.h"
#if (BLE_UART_TP_SERVER)
//#if 1
#include <stdint.h>
#include "rwip_task.h" // Task definitions
/*
 * DEFINES
 ****************************************************************************************
 */

///Maximum number of UART transparent Server task instances
#define UART_TP_IDX_MAX      0x01
///Maximal number of UART transparent that can be added in the DB

#define UART_TX_PAYLOAD_LEN  244
#define UART_RX_PAYLOAD_LEN  244
/*
 * TYPE DEFINITIONS
 ****************************************************************************************
 */

/// Possible states of the UART transparent task
enum uart_tp_state
{
    /// Idle state
    UART_TP_IDLE,
    /// busy state
    UART_TP_BUSY,
    /// Number of defined states.
    UART_TP_STATE_MAX
};

/// Messages for UART transparent Server
enum uart_tp_msg_id
{
    /// Start the UART transparent Server - at connection used to restore bond data
    UART_TP_CREATE_DB_REQ   = TASK_FIRST_MSG(TASK_ID_UART_TP),
	
    /// UART Tx Update Request
    UART_TX_PAYLOAD_UPD_REQ,
    /// Inform APP if UART Tx value has been notified or not
    UART_TX_PAYLOAD_UPD_RSP,
    /// Inform APP that UART Tx Notification Configuration has been changed - use to update bond data
    UART_TX_PAYLOAD_NTF_CFG_IND,
	
    UART_RX_WRITER_REQ_IND,

    UART_TX_PAYLOAD_PERIOD_NTF
	
		
};

/// Features Flag Masks
enum uart_tp_features
{
    /// UART Tx Characteristic doesn't support notifications
    UART_TP_TX_NTF_NOT_SUP,
    /// UART Tx Characteristic support notifications
    UART_TP_TX_NTF_SUP,
};
/*
 * APIs Structures
 ****************************************************************************************
 */

/// Parameters for the database creation
struct uart_tp_db_cfg
{
    /// Number of UART transparent to add
    uint8_t uart_tp_nb;
    /// Features of each UART transparent instance
    uint8_t features;
   };

/// Parameters of the @ref UART_TP_ENABLE_REQ message
struct uart_tp_enable_req
{
    /// connection index
    uint8_t  conidx;
    /// Notification Configuration
    uint8_t  ntf_cfg;
    /// Old UART Tx used to decide if notification should be triggered
    uint8_t  old_tx_payload;
};


///Parameters of the @ref F000_BATT_LEVEL_UPD_REQ message
struct uart_tx_payload_upd_req
{
    /// BAS instance
    uint8_t conidx;
	
    uint8_t length;
    /// fff1 Level
    uint8_t tx_payload[UART_TX_PAYLOAD_LEN];
};

///Parameters of the @ref UART_TX_PAYLOAD_UPD_RSP message
struct uart_tx_payload_upd_rsp
{
    ///status
    uint8_t status;
};

///Parameters of the @ref BASS_BATT_LEVEL_NTF_CFG_IND message
struct uart_tx_payload_ntf_cfg_ind
{
    /// connection index
    uint8_t  conidx;
    ///Notification Configuration
    uint8_t  ntf_cfg;
};


/// Parameters of the @ref UART_RX_WRITER_REQ_IND message
struct uart_rx_writer_ind
{
    /// Alert level
    uint8_t uart_rx_payload[UART_RX_PAYLOAD_LEN];
	
    uint8_t length;
    /// Connection index
    uint8_t conidx;
};


/*
 * TASK DESCRIPTOR DECLARATIONS
 ****************************************************************************************
 */

extern const struct ke_state_handler uart_tp_default_handler;
#endif // BLE_UART_TP_SERVER


#endif /* _UART_TP_TASK_H_ */

