/**
****************************************************************************************
*
* @file icu.c
*
* @brief icu initialization and specific functions
*
* Copyright (C) Beken 2009-2016
*
* $Rev: $
*
****************************************************************************************
*/

/**
****************************************************************************************
* @addtogroup ICU
* @ingroup ICU
* @brief ICU
*
* This is the driver block for ICU
* @{
****************************************************************************************
*/


#include <stddef.h>     // standard definition
#include "rwip_config.h"
#include "user_config.h"
#include "BK3432_reg.h"
#include "icu.h"      // timer definition
#include "rf.h"
#include "uart.h"



static volatile uint8_t reduce_voltage_set=0;
static volatile uint8_t default_sleep_mode = 0;  //0:ѹ   1:ͨidel
extern uint8_t system_sleep_flag;

void system_sleep_init(void)
{
#if SYSTEM_SLEEP
	system_sleep_flag = 0x1;
#else
	system_sleep_flag = 0x0;
#endif
}
void icu_init(void)
{
#if DEBUG_HW
	REG_AHB0_ICU_LPO_CLK_ON |= (0x01 << 2);
#endif

	REG_AHB0_ICU_FLASH &= ~(0xff << 16);
	REG_AHB0_ICU_FLASH |= (0x15 << 16);

	REG_AHB0_ICU_CPU_STATUS  = 0x661;


	REG_AHB0_ICU_DIGITAL_PWD = REG_AHB0_ICU_DIGITAL_PWD & (~0X02);

	REG_AHB0_ICU_CORECLKCON = 0X00;

	REG_AHB0_ICU_CLKSRCSEL = 0X000001b1; //usr 16M,

	REG_AHB0_ICU_ANA_CTL |= (0X01 << 6);

}

uint8_t icu_get_sleep_mode(void)
{
	return default_sleep_mode;
}


void icu_set_sleep_mode(uint8_t v)
{
	default_sleep_mode = v;
}

void enter_deep_sleep(void)
{   
    REG_AHB0_ICU_INT_ENABLE = 0;
    REG_AHB0_ICU_IRQ_ENABLE = 0;
    
    REG_AHB0_ICU_RC32K_CONFG = 0x5a;
    
    REG_AHB0_ICU_DEEP_SLEEP1|=( 0x3432<<16);
    Delay_us(10);
    while(1);
}

#if SYSTEM_SLEEP
void cpu_reduce_voltage_sleep(void)
{
	reduce_voltage_set=1;
    
	REG_AHB0_ICU_DIGITAL_PWD |= 0x80;


	REG_AHB0_ICU_CLKSRCSEL = 0X000001b0;
//    //charles add
//    REG_AHB0_ICU_CLKSRCSEL  = 0x184;// 0x184
//	REG_AHB0_ICU_CPU_STATUS  = 0x611;  // spi vol		
//    REG_AHB0_ICU_CLKSRCSEL |= 0x04;//0x04 low voltage
//    REG_AHB0_ICU_CORECLKCON = 0x0 ; // CPU is power-up and cpu-clock is not divided. 

    ////////////////////////////////////////////////////////
 
   // XVR_REG09 = XVR_ANALOG_REG_BAK[0x09] |(1<<27);
	REG_AHB0_ICU_DIGITAL_PWD |= SET_CLK48M_PWD;
	REG_AHB0_ICU_DIGITAL_PWD |= SET_CB_BAIS_PWD;
	REG_AHB0_ICU_DIGITAL_PWD |= SET_CLK16M_PWD;//can not power dn 16M, or flash cant nor be read
	REG_AHB0_ICU_DIGITAL_PWD |=SET_HP_LDO_PWD;
	REG_AHB0_ICU_CORECLKCON  = 0x1; // Power down MCU
}


void cpu_wakeup(void)
{
	if(reduce_voltage_set==1)
	{
		reduce_voltage_set=0;
		//REG_AHB0_ICU_INT_ENABLE |= (0x01 << 9);

		REG_AHB0_ICU_DIGITAL_PWD = 0x80;
		REG_AHB0_ICU_CLKSRCSEL  = 0x1b1;

	}
}



void cpu_idle_sleep(void)
{
	REG_AHB0_ICU_CORECLKCON = 0x01;//MCU sleep
}


#endif



void switch_clk(uint8_t clk)
{
	REG_AHB0_ICU_DIGITAL_PWD = REG_AHB0_ICU_DIGITAL_PWD & (~0X02);

	if (clk == MCU_CLK_16M )
	{
		REG_AHB0_ICU_CORECLKCON = 0X00; //clk div 0

		REG_AHB0_ICU_CLKSRCSEL = 0X000001b1; //usr 16m,VN=7,VD=3

	}
	else if (clk == MCU_CLK_64M )
	{
		REG_AHB0_ICU_CORECLKCON = 0X00; //clk div 0

		REG_AHB0_ICU_CLKSRCSEL = 0X000005b3; //usr PLL CLK SELT 64M

	}

	else if (clk == MCU_CLK_32M )
	{
		REG_AHB0_ICU_CORECLKCON = 0x04; //clk div 2

		REG_AHB0_ICU_CLKSRCSEL = 0X000005b3; //usr PLL CLK SELT 64M

	}
}



//õز
//freq:Ƶã˫Ƶ(2-80)
//power:ʵȼ(0x1-0xf)
void bk3435_singleWaveCfg(uint8_t freq, uint8_t power_level)
{
	uint32_t val = 0;
    uint16_t count = 0;

    //XVR_REG24  &= ~(0x1 << 20);
	val |= freq;
	val |= (power_level<< 7);
   // XVR_REG04 &= ~(0x01<<9);
	XVR_REG24 = val;
    for(count = 0;count<100;count++)
    {
         UART_PRINTF("XVR24 : %x \r\n",XVR_REG24);
    }
    XVR_REG25 &= 0xFFF00000 ;  // Clear XVR Test    //clear [19:0] 
    for(count = 0;count<100;count++)
    {
         UART_PRINTF("XVR25 : %x \r\n",XVR_REG25);
    }
	XVR_REG25 |= (0x1<<12) |(0x1<<13);
    for(count = 0;count<100;count++)
    {
         UART_PRINTF("XVR25 : %x \r\n",XVR_REG25);
    }
	
  
    UART_PRINTF("%s\r\n",__func__);
    UART_PRINTF("XVR24 : %x \r\n",XVR_REG24);
    UART_PRINTF("XVR25 : %x \r\n",XVR_REG25);

	while(1);
}



